package com.sun.org.apache.xml.internal.security.c14n.implementations;

import java.security.AccessController;
import java.io.IOException;
import java.util.Map;
import java.io.OutputStream;

public final class UtfHelpper
{
    private static final boolean OLD_UTF8;
    
    private UtfHelpper() {
    }
    
    public static void writeByte(String s, OutputStream outputStream, Map<String, byte[]> map) throws IOException {
        byte[] stringInUtf8 = (byte[])map.get(s);
        if (stringInUtf8 == null) {
            stringInUtf8 = getStringInUtf8(s);
            map.put(s, stringInUtf8);
        }
        outputStream.write(stringInUtf8);
    }
    
    public static void writeCodePointToUtf8(int codePoint, OutputStream outputStream) throws IOException {
        if (!Character.isValidCodePoint(codePoint) || (codePoint >= 55296 && codePoint <= 56319) || (codePoint >= 56320 && codePoint <= 57343)) {
            outputStream.write(63);
            return;
        }
        if (UtfHelpper.OLD_UTF8 && codePoint >= 65536) {
            outputStream.write(63);
            outputStream.write(63);
            return;
        }
        if (codePoint < 128) {
            outputStream.write(codePoint);
            return;
        }
        int n;
        if (codePoint < 2048) {
            n = 1;
        }
        else if (codePoint < 65536) {
            n = 2;
        }
        else if (codePoint < 2097152) {
            n = 3;
        }
        else if (codePoint < 67108864) {
            n = 4;
        }
        else if (codePoint <= Integer.MAX_VALUE) {
            n = 5;
        }
        else {
            outputStream.write(63);
            return;
        }
        int n2 = 6 * n;
        outputStream.write((int)(byte)(254 << 6 - n | codePoint >>> n2));
        for (int i = n - 1; i >= 0; --i) {
            n2 -= 6;
            outputStream.write((int)(byte)(0x80 | (codePoint >>> n2 & 0x3F)));
        }
        return;
    }
    
    @Deprecated
    public static void writeCharToUtf8(char c, OutputStream outputStream) throws IOException {
        if (c < '\u0080') {
            outputStream.write((int)c);
            return;
        }
        if ((c >= '?' && c <= '?') || (c >= '?' && c <= '?')) {
            outputStream.write(63);
            return;
        }
        int n;
        int n2;
        if (c > '߿') {
            int n3 = (char)(c >>> 12);
            int n4 = 224;
            if (n3 > '\0') {
                n4 |= (n3 & '\u000f');
            }
            outputStream.write(n4);
            n = 128;
            n2 = '?';
        }
        else {
            n = 192;
            n2 = '\u001f';
        }
        int n5 = (char)(c >>> 6);
        if (n5 > '\0') {
            n |= (n5 & n2);
        }
        outputStream.write(n);
        outputStream.write(0x80 | (c & '?'));
        return;
    }
    
    public static void writeStringToUtf8(String s, OutputStream outputStream) throws IOException {
        int length = s.length();
        int i = 0;
        while (i < length) {
            int codePoint = s.codePointAt(i);
            i += Character.charCount(codePoint);
            if (!Character.isValidCodePoint(codePoint) || (codePoint >= 55296 && codePoint <= 56319) || (codePoint >= 56320 && codePoint <= 57343)) {
                outputStream.write(63);
            }
            else if (UtfHelpper.OLD_UTF8 && codePoint >= 65536) {
                outputStream.write(63);
                outputStream.write(63);
            }
            else if (codePoint < 128) {
                outputStream.write(codePoint);
            }
            else {
                int n;
                if (codePoint < 2048) {
                    n = 1;
                }
                else if (codePoint < 65536) {
                    n = 2;
                }
                else if (codePoint < 2097152) {
                    n = 3;
                }
                else if (codePoint < 67108864) {
                    n = 4;
                }
                else if (codePoint <= Integer.MAX_VALUE) {
                    n = 5;
                }
                else {
                    outputStream.write(63);
                    continue;
                }
                int n2 = 6 * n;
                outputStream.write((int)(byte)(254 << 6 - n | codePoint >>> n2));
                for (int j = n - 1; j >= 0; --j) {
                    n2 -= 6;
                    outputStream.write((int)(byte)(0x80 | (codePoint >>> n2 & 0x3F)));
                }
            }
        }
    }
    
    public static byte[] getStringInUtf8(String s) {
        int length = s.length();
        int n = 0;
        byte[] array = new byte[length];
        int i = 0;
        int n2 = 0;
        while (i < length) {
            int codePoint = s.codePointAt(i);
            i += Character.charCount(codePoint);
            if (!Character.isValidCodePoint(codePoint) || (codePoint >= 55296 && codePoint <= 56319) || (codePoint >= 56320 && codePoint <= 57343)) {
                array[n2++] = 63;
            }
            else if (UtfHelpper.OLD_UTF8 && codePoint >= 65536) {
                array[n2++] = 63;
                array[n2++] = 63;
            }
            else if (codePoint < 128) {
                array[n2++] = (byte)codePoint;
            }
            else {
                if (n == 0) {
                    byte[] array2 = new byte[6 * length];
                    System.arraycopy(array, 0, array2, 0, n2);
                    array = array2;
                    n = 1;
                }
                int n3;
                if (codePoint < 2048) {
                    n3 = 1;
                }
                else if (codePoint < 65536) {
                    n3 = 2;
                }
                else if (codePoint < 2097152) {
                    n3 = 3;
                }
                else if (codePoint < 67108864) {
                    n3 = 4;
                }
                else if (codePoint <= Integer.MAX_VALUE) {
                    n3 = 5;
                }
                else {
                    array[n2++] = 63;
                    continue;
                }
                int n4 = 6 * n3;
                array[n2++] = (byte)(254 << 6 - n3 | codePoint >>> n4);
                for (int j = n3 - 1; j >= 0; --j) {
                    n4 -= 6;
                    array[n2++] = (byte)(0x80 | (codePoint >>> n4 & 0x3F));
                }
            }
        }
        if (n != 0) {
            byte[] array3 = new byte[n2];
            System.arraycopy(array, 0, array3, 0, n2);
            array = array3;
            return array;
        }
        return array;
    }
    
    static {
        OLD_UTF8 = (boolean)AccessController.doPrivileged(()/*  */ -> Boolean.valueOf(Boolean.getBoolean("com.sun.org.apache.xml.internal.security.c14n.oldUtf8")));
    }
}
